import importlib
import logging

from logtron.config import discover_config
from logtron.formatters import JsonFormatter

is_configured = False


def autodiscover(name=None, level=logging.INFO, config=None, refresh=False, discover_context=None):
    global is_configured

    if not refresh and is_configured:
        return logging.getLogger(name)

    config = discover_config(config)

    root_logger = logging.getLogger()
    root_logger.setLevel(level)
    existing_handlers = root_logger.handlers
    [root_logger.removeHandler(i) for i in existing_handlers]

    context = {}
    if "context" in config:
        context = config["context"]
    elif discover_context is not None:
        context = discover_context()

    for i in config["handlers"]:
        module_name, class_name = i.rsplit(".", 1)
        HandlerClass = getattr(importlib.import_module(module_name), class_name)
        instance = None
        if i in config and config[i] is not None:
            instance = HandlerClass(context, **config[i])
        else:
            instance = HandlerClass(context)
        instance.setFormatter(JsonFormatter(context))
        root_logger.addHandler(instance)

    is_configured = True

    return logging.getLogger(name)



import json
import traceback
from logging import Formatter, LogRecord


class JsonFormatter(Formatter):
    def __init__(self, context=None):
        super(JsonFormatter, self).__init__()
        self.context = context
        self.dummy = LogRecord(None, None, None, None, None, None, None)

    def format(self, record):
        data = {
            "timestamp": int(record.created * 1000),
            "message": record.msg,
            "name": record.name,
            "level": record.levelno,
            "context": self.context,
        }

        data["extra"] = {}
        for k, v in record.__dict__.items():
            if k not in self.dummy.__dict__ and k not in ["message"]:
                data["extra"][k] = v

        if record.exc_info is not None:
            data["exception"] = "".join(traceback.format_exception(*record.exc_info))

        return json.dumps(data)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['logtron', 'logtron.config', 'logtron.formatters', 'logtron.handlers']

package_data = \
{'': ['*']}

install_requires = \
['importlib-metadata>=1.7.0,<2.0.0', 'pyyaml>=5.3.1,<6.0.0']

setup_kwargs = {
    'name': 'logtron',
    'version': '0.1.6',
    'description': 'A simple logging library with JSON log formatting',
    'long_description': '# Logtron\n\n**Logtron** is a simple logging library with JSON log formatting.\n\n```python\n>>> import logtron\n>>> logger = logtron.autodiscover()\n>>> logger.info("hello world")\n{"timestamp": 1598900664859, "message": "hello world", "name": "root", "level": 20, "context": {}, "extra": {}}\n>>> logger.info("extra args", extra={"foo": "bar", "count": 7})\n{"timestamp": 1598900667704, "message": "extra args", "name": "root", "level": 20, "context": {}, "extra": {"foo": "bar", "count": 7}}\n>>>\n```\n\nOr\n\n```python\n>>> import logtron\n>>> logtron.autodiscover() # Only needs to run once somewhere to configure the root logger\n<RootLogger root (INFO)>\n>>>\n>>> import logging\n>>> logger = logging.getLogger()\n>>> logger.info("hello world")\n{"timestamp": 1598900735699, "message": "hello world", "name": "root", "level": 20, "context": {}, "extra": {}}\n>>> logger.info("extra args", extra={"foo": "bar", "count": 7})\n{"timestamp": 1598900757238, "message": "extra args", "name": "root", "level": 20, "context": {}, "extra": {"foo": "bar", "count": 7}}\n>>>\n```\n\nLogtron allows you to skip all the usual boilerplate when configuring python logging.\n\nLogtron will default to a console JSON log formatter that is compatible with popular log aggregators such as [Logstash](https://www.elastic.co/guide/en/logstash/current/introduction.html), [Fluent Bit](https://docs.fluentbit.io/manual/), or [AWS CloudWatch Logs](https://docs.aws.amazon.com/AmazonCloudWatch/latest/logs/WhatIsCloudWatchLogs.html).\n\n[![Downloads](https://pepy.tech/badge/logtron/month)](https://pepy.tech/project/logtron/month)\n[![Supported Versions](https://img.shields.io/pypi/pyversions/logtron.svg)](https://pypi.org/project/logtron)\n[![Contributors](https://img.shields.io/github/contributors/ilija1/logtron.svg)](https://github.com/ilija1/logtron/graphs/contributors)\n\n[![Build Status](https://travis-ci.org/ilija1/logtron.svg?branch=master)](https://travis-ci.org/ilija1/logtron)\n[![codecov](https://codecov.io/gh/ilija1/logtron/branch/master/graph/badge.svg)](https://codecov.io/gh/ilija1/logtron)\n[![Documentation Status](https://readthedocs.org/projects/logtron/badge/?version=latest)](https://logtron.readthedocs.io/en/latest/?badge=latest)\n\n## Installing Logtron and Supported Versions\n\nLogtron is available on PyPI:\n\n```shell\n$ python -m pip install logtron\n```\n\nLogtron officially supports Python 2.7 & 3.5+.\n',
    'author': 'Ilija Stevcev',
    'author_email': 'ilija1@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ilija1/logtron/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*',
}


setup(**setup_kwargs)



import os

import yaml

DEFAULT_CONFIG = {
    "handlers": [
        "logtron.handlers.ConsoleHandler",
    ],
}


def _parse_value(value):
    if type(value) == list:
        return [_parse_value(i) for i in value]

    if value.isdigit():
        return int(value)
    try:
        return float(value)
    except ValueError:
        pass
    return value


def _parse_nested_env(key, value):
    split = key.split("_")
    if len(split) == 1:
        if "," in value:
            value = value.split(",")
        return {key: _parse_value(value)}
    return {
        split[0]: _parse_nested_env("_".join(split[1:]), value),
    }


def _parse_env():
    env_config = dict(
        [
            (i.lower().split("logtron_")[-1], os.environ.get(i))
            for i in os.environ.keys()
            if i.lower().startswith("logtron_")
        ]
    )
    env_config_parsed = {}
    for k, v in env_config.items():
        nested = _parse_nested_env(k, v)
        env_config_parsed.update(nested)

    return env_config_parsed


def discover_config(existing=None):
    config = DEFAULT_CONFIG.copy()
    config_file = "logtron.yaml"

    if existing is not None:
        if type(existing) == dict:
            config.update(existing)
        elif type(existing) == str:
            config_file = existing

    # Read config file
    if os.path.isfile(config_file):
        f = open(config_file, "r")
        file_config = yaml.load(f, Loader=yaml.SafeLoader)
        f.close()
        config.update(file_config)

    # Read env vars
    env_config = _parse_env()

    config.update(env_config)

    return config
